@@ -726,7 +726,7 @@ scm_basic_basic_make_class (SCM class, SCM name, SCM dsupers, SCM dslots)
   SCM_SLOT(z, scm_si_getters_n_setters) = g_n_s;
   SCM_SLOT(z, scm_si_redefined)         = SCM_BOOL_F;
   SCM_SLOT(z, scm_si_environment)
-    = scm_top_level_env (SCM_CDR (scm_top_level_lookup_closure_var));
+    = scm_top_level_env (SCM_TOP_LEVEL_LOOKUP_CLOSURE);
 
   /* Add this class in the direct-subclasses slot of dsupers */
   {
@@ -845,7 +845,8 @@ create_basic_classes (void)
   /* SCM_SLOT(scm_class_class, scm_si_getters_n_setters)
      = compute_getters_n_setters (slots_of_class); */
   SCM_SLOT(scm_class_class, scm_si_redefined) 	 = SCM_BOOL_F;
-  SCM_SLOT(scm_class_class, scm_si_environment) = scm_top_level_env (SCM_CDR (scm_top_level_lookup_closure_var));
+  SCM_SLOT(scm_class_class, scm_si_environment)
+    = scm_top_level_env (SCM_TOP_LEVEL_LOOKUP_CLOSURE);
 
   prep_hashsets (scm_class_class);
 
@@ -1757,8 +1758,12 @@ scm_sys_invalidate_method_cache_x (SCM gf)
 	   methods = SCM_CDR (methods))
 	SCM_SLOT (SCM_CAR (methods), scm_si_code_table) = SCM_EOL;
     }
-  SCM_SET_MCACHE_N_SPECIALIZED (SCM_ENTITY_PROCEDURE (gf),
-				SCM_SLOT (gf, scm_si_n_specialized));
+  {
+    int n = SCM_INUM (SCM_SLOT (gf, scm_si_n_specialized));
+    /* The sign of n is a flag indicating rest args. */
+    SCM_SET_MCACHE_N_SPECIALIZED (SCM_ENTITY_PROCEDURE (gf),
+				  SCM_MAKINUM (n >= 0 ? n : -n));
+  }
   return SCM_UNSPECIFIED;
 }
 
@@ -2506,7 +2511,15 @@ make_class_from_template (char *template, char *type_name, SCM supers)
       && SCM_FALSEP (scm_apply (scm_goops_lookup_closure,
 				SCM_LIST2 (name, SCM_BOOL_F),
 				SCM_EOL)))
-    DEFVAR (name, class);
+    {
+      /* Make sure we add the binding in the GOOPS module.
+       * This kludge is needed until DEFVAR ceases to use `define-public'
+       * or `define-public' ceases to use `current-module'.
+       */
+      SCM old_module = scm_select_module (scm_module_goops);
+      DEFVAR (name, class);
+      scm_select_module (old_module);
+    }
   return class;
 }
 
@@ -2752,7 +2765,7 @@ scm_wrap_component (SCM class, SCM container, void *data)
 SCM
 scm_ensure_accessor (SCM name)
 {
-  SCM gf = scm_apply (SCM_CDR (scm_top_level_lookup_closure_var),
+  SCM gf = scm_apply (SCM_TOP_LEVEL_LOOKUP_CLOSURE,
 		      SCM_LIST2 (name, SCM_BOOL_F),
 		      SCM_EOL);
   if (!SCM_IS_A_P (gf, scm_class_generic_with_setter))
@@ -2809,18 +2822,21 @@ sys_goops_loaded ()
   return SCM_UNSPECIFIED;
 }
 
+SCM scm_module_goops;
+
 void
 scm_init_goops (void)
 {
-  SCM goops_module = scm_make_module (scm_read_0str ("(oop goops)"));
-  SCM old_module = scm_select_module (goops_module);
+  SCM old_module;
 #ifndef SCM_UNBOUND
   SCM the_unbound_value = SCM_CAR (scm_intern0 ("the-unbound-value"));
   scm_goops_the_unbound_value
     = scm_permanent_object (scm_cons (the_unbound_value, SCM_EOL));
 #endif
+  scm_module_goops = scm_make_module (scm_read_0str ("(oop goops)"));
+  old_module = scm_select_module (scm_module_goops);
   
-  scm_goops_lookup_closure = scm_module_lookup_closure (goops_module);
+  scm_goops_lookup_closure = scm_module_lookup_closure (scm_module_goops);
 
   scm_components = scm_permanent_object (scm_make_weak_key_hash_table
 					 (SCM_MAKINUM (37)));
